@@ -37,7 +37,7 @@
 #include <eel/eel-string.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-url.h>
@@ -121,7 +121,7 @@ application_cannot_open_location (GnomeVFSMimeApplication *application,
 				  const char *uri_scheme,
 				  GtkWindow *parent_window)
 {
-#if NEW_MIME_COMPLETE
+#ifdef NEW_MIME_COMPLETE
 	GtkDialog *message_dialog;
 	LaunchParameters *launch_parameters;
 	char *prompt;
@@ -147,7 +147,6 @@ application_cannot_open_location (GnomeVFSMimeApplication *application,
 		
 		message_dialog = eel_show_yes_no_dialog (prompt, 
 		                                         message,
-							 _("Can't Open Location"), 
 							 GTK_STOCK_OK,
 							 GTK_STOCK_CANCEL,
 							 parent_window);
@@ -166,21 +165,21 @@ application_cannot_open_location (GnomeVFSMimeApplication *application,
 		g_free (message);
 	} else {
 		if (application != NULL) {
-			prompt = g_strdup_printf (_("\"%s\" can't open \"%s\" because \"%s\" can't access files at \"%s\"."
+			prompt = g_strdup_printf (_("\"%s\" can't open \"%s\" because \"%s\" can't access files at \"%s\" "
 						    "locations."), application->name, file_name, 
 						    application->name, uri_scheme);
 			message = _("No other applications are available to view this file.  "
 				    "If you copy this file onto your computer, you may be able to open "
 				    "it.");
 		} else {
-			prompt = g_strdup_printf (_("The default action can't open \"%s\" because it can't access files at \"%s\"."
+			prompt = g_strdup_printf (_("The default action can't open \"%s\" because it can't access files at \"%s\" "
 						    "locations."), file_name, uri_scheme);
      			message = _("No other actions are available to view this file.  "
 				    "If you copy this file onto your computer, you may be able to open "
 				    "it.");
 		}
 				
-		eel_show_info_dialog (prompt, message, _("Can't Open Location"), parent_window);
+		eel_show_info_dialog (prompt, message, parent_window);
 		g_free (prompt);
 	}	
 
@@ -446,8 +445,6 @@ void nautilus_launch_show_file (NautilusFile *file,
 	startup_notify = FALSE;
 #endif
 
-	g_return_if_fail (!nautilus_file_needs_slow_mime_type (file));
-
 	uri = NULL;
 	if (nautilus_file_is_nautilus_link (file)) {
 		uri = nautilus_file_get_activation_uri (file);
@@ -572,7 +569,7 @@ void nautilus_launch_show_file (NautilusFile *file,
 		
 	case GNOME_VFS_ERROR_NO_DEFAULT:
 	case GNOME_VFS_ERROR_NO_HANDLER:
-#if NEW_MIME_COMPLETE
+#ifdef NEW_MIME_COMPLETE
 		nautilus_program_chooser_show_no_choices_message
 					(action_type, file, parent_window);
 		break;
@@ -632,7 +629,7 @@ void nautilus_launch_show_file (NautilusFile *file,
 			break;
 		case GNOME_VFS_OK:
 		default:
-#if NEW_MIME_COMPLETE
+#ifdef NEW_MIME_COMPLETE
 			nautilus_program_chooser_show_invalid_message
 				(action_type, file, parent_window);
 #endif
@@ -665,18 +662,19 @@ void nautilus_launch_show_file (NautilusFile *file,
  * parameter. Provide a parent window for error dialogs. 
  * 
  * @application: The application to be launched.
- * @file: The file whose location should be passed as a parameter to the application
+ * @files: The files whose locations should be passed as a parameter to the application.
  * @parent_window: A window to use as the parent for any error dialogs.
  */
 void
 nautilus_launch_application (GnomeVFSMimeApplication *application, 
-			     NautilusFile *file,
+			     GList *files,
 			     GtkWindow *parent_window)
 {
 	GdkScreen       *screen;
 	char		*uri;
 	char            *uri_scheme;
-	GList            uris;
+	GList           *uris, *l;
+	NautilusFile    *file;
 	char           **envp;
 	GnomeVFSResult   result;
 #ifdef HAVE_STARTUP_NOTIFICATION
@@ -684,19 +682,26 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 	SnDisplay *sn_display;
 #endif
 
-	uri = NULL;
-	if (nautilus_file_is_nautilus_link (file)) {
-		uri = nautilus_file_get_activation_uri (file);
+	g_assert (files != NULL);
+
+	uris = NULL;
+	for (l = files; l != NULL; l = l->next) {
+		file = NAUTILUS_FILE (l->data);
+
+		uri = NULL;
+
+		if (nautilus_file_is_nautilus_link (file)) {
+			uri = nautilus_file_get_activation_uri (file);
+		}
+		
+		if (uri == NULL) {
+			uri = nautilus_file_get_uri (file);
+		}
+
+		uris = g_list_prepend (uris, uri);
 	}
-	
-	if (uri == NULL) {
-		uri = nautilus_file_get_uri (file);
-	}
+	uris = g_list_reverse (uris);
 
-	uris.next = NULL;
-	uris.prev = NULL;
-	uris.data = uri;
-	
 	screen = gtk_window_get_screen (parent_window);
 	envp = my_gdk_spawn_make_environment_for_screen (screen, NULL);
 	
@@ -710,27 +715,44 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 	if (gnome_vfs_mime_application_supports_startup_notification (application))
 	{ 
 		char *name;
+		char *description;
 		char *icon;
+		int   files_count;
+
+		file = NAUTILUS_FILE (files->data);
 
 		sn_context = sn_launcher_context_new (sn_display,
 						      screen ? gdk_screen_get_number (screen) :
 						      DefaultScreen (gdk_display));
-		
-		name = nautilus_file_get_display_name (file);
-		if (name != NULL) {
-			char *description;
-			
-			sn_launcher_context_set_name (sn_context, name);
-			
-			description = g_strdup_printf (_("Opening %s"), name);
-			
-			sn_launcher_context_set_description (sn_context, description);
 
+		files_count = g_list_length (files);
+		if (files_count == 1) {
+			name = nautilus_file_get_display_name (file);
+			description = g_strdup_printf (_("Opening %s"), name);
+		} else {
+			name = NULL;
+			description = g_strdup_printf (ngettext ("Opening %d Item",
+								 "Opening %d Items",
+								 files_count),
+						       files_count);
+		}
+
+		if (name != NULL) {
+			sn_launcher_context_set_name (sn_context, name);
 			g_free (name);
+		}
+
+		if (description != NULL) {
+			sn_launcher_context_set_description (sn_context, description);
 			g_free (description);
 		}
 
 		icon = nautilus_icon_factory_get_icon_for_file (file, FALSE);
+
+		if (icon == NULL) {
+			icon = g_strdup (gnome_vfs_mime_application_get_icon (application));
+		}
+
 		if (icon != NULL) {
 			sn_launcher_context_set_icon_name (sn_context, icon);
 			g_free (icon);
@@ -762,7 +784,7 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 	}
 #endif /* HAVE_STARTUP_NOTIFICATION */
 	
-	result = gnome_vfs_mime_application_launch_with_env (application, &uris, envp);
+	result = gnome_vfs_mime_application_launch_with_env (application, uris, envp);
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 	if (sn_context != NULL) {
@@ -784,7 +806,7 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 		break;
 
 	case GNOME_VFS_ERROR_NOT_SUPPORTED:
-		uri_scheme = nautilus_file_get_uri_scheme (file);
+		uri_scheme = nautilus_file_get_uri_scheme (NAUTILUS_FILE (files->data));
 		application_cannot_open_location (application,
 						  file,
 						  uri_scheme,
@@ -794,15 +816,15 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 		break;
 
 	default:
-#if NEW_MIME_COMPLETE
+#ifdef NEW_MIME_COMPLETE
 		nautilus_program_chooser_show_invalid_message
 			(GNOME_VFS_MIME_ACTION_TYPE_APPLICATION, file, parent_window);
 			 
 #endif
 		break;
 	}
-	
-	g_free (uri);
+
+	eel_g_list_free_deep (uris);
 	g_strfreev (envp);
 }
 
